package org.commonjava.tensor.data;

import java.net.URI;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.commonjava.maven.atlas.common.ref.ProjectRef;
import org.commonjava.maven.atlas.common.ref.ProjectVersionRef;
import org.commonjava.maven.atlas.effective.EProjectDirectRelationships;
import org.commonjava.maven.atlas.effective.EProjectGraph;
import org.commonjava.maven.atlas.effective.EProjectWeb;
import org.commonjava.maven.atlas.effective.filter.ProjectRelationshipFilter;
import org.commonjava.maven.atlas.effective.ref.EProjectKey;
import org.commonjava.maven.atlas.effective.rel.ProjectRelationship;
import org.commonjava.maven.atlas.effective.workspace.GraphWorkspace;
import org.commonjava.maven.atlas.effective.workspace.GraphWorkspaceConfiguration;

public interface TensorDataManager
{

    String MODEL_ERRORS = "tensor-modelErrors";

    String ERROR_SEPARATOR = Pattern.quote( "_::--::_" );

    Set<ProjectRelationship<?>> storeGraph( EProjectGraph graph )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( EProjectDirectRelationships relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( ProjectRelationship<?>... relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( Collection<ProjectRelationship<?>> relationships )
        throws TensorDataException;

    EProjectGraph getProjectGraph( ProjectRelationshipFilter filter, ProjectVersionRef discovered )
        throws TensorDataException;

    EProjectGraph getProjectGraph( ProjectVersionRef ref )
        throws TensorDataException;

    List<ProjectVersionRef> getAncestry( ProjectVersionRef source )
        throws TensorDataException;

    ProjectVersionRef getParent( ProjectVersionRef source )
        throws TensorDataException;

    Set<ProjectVersionRef> getKnownChildren( ProjectVersionRef parent )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactSource( ProjectVersionRef source,
                                                                          ProjectRelationshipFilter filter )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactTarget( ProjectVersionRef target,
                                                                          ProjectRelationshipFilter filter )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithGASource( ProjectRef source,
                                                                       ProjectRelationshipFilter filter )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithGATarget( ProjectRef target,
                                                                       ProjectRelationshipFilter filter )
        throws TensorDataException;

    boolean contains( ProjectVersionRef ref );

    Set<ProjectVersionRef> getAllStoredProjectRefs()
        throws TensorDataException;

    Map<String, String> getMetadata( ProjectVersionRef ref )
        throws TensorDataException;

    void addMetadata( EProjectKey key, String name, String value );

    void addMetadata( EProjectKey key, Map<String, String> metadata );

    Set<ProjectVersionRef> getIncompleteSubgraphsFor( ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getIncompleteSubgraphsFor( ProjectRelationshipFilter filter, ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getVariableSubgraphsFor( ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getVariableSubgraphsFor( ProjectRelationshipFilter filter, ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getAllIncompleteSubgraphs()
        throws TensorDataException;

    Set<ProjectVersionRef> getAllVariableSubgraphs()
        throws TensorDataException;

    void addError( final EProjectKey key, final Throwable error )
        throws TensorDataException;

    Set<String> getErrors( final ProjectVersionRef ref )
        throws TensorDataException;

    boolean hasErrors( final ProjectVersionRef ref )
        throws TensorDataException;

    Map<ProjectVersionRef, Set<String>> getAllProjectErrors()
        throws TensorDataException;

    Map<ProjectVersionRef, Set<String>> getProjectErrorsInGraph( ProjectVersionRef ref )
        throws TensorDataException;

    void reindex( ProjectVersionRef ref )
        throws TensorDataException;

    void reindexAll()
        throws TensorDataException;

    EProjectWeb getProjectWeb( ProjectRelationshipFilter filter, ProjectVersionRef... refs )
        throws TensorDataException;

    EProjectWeb getProjectWeb( ProjectVersionRef... refs )
        throws TensorDataException;

    Set<ProjectVersionRef> pathFilter( ProjectRelationshipFilter filter, Set<ProjectVersionRef> leaves,
                                       ProjectVersionRef... roots )
        throws TensorDataException;

    GraphWorkspace createWorkspace( URI sourceUri )
        throws TensorDataException;

    GraphWorkspace createWorkspace( GraphWorkspaceConfiguration config )
        throws TensorDataException;

    GraphWorkspace createTemporaryWorkspace( GraphWorkspaceConfiguration config )
        throws TensorDataException;

    GraphWorkspace setCurrentWorkspace( String sessionId )
        throws TensorDataException;

    Set<ProjectVersionRef> getMatchingGAVs( ProjectRef projectRef )
        throws TensorDataException;

    boolean deleteWorkspace( String id );

    Set<GraphWorkspace> getAllWorkspaces();

    GraphWorkspace getCurrentWorkspace();

    void clearCurrentWorkspace()
        throws TensorDataException;

}