package org.commonjava.tensor.io.json;

import java.lang.reflect.Type;
import java.net.URI;
import java.net.URISyntaxException;

import org.commonjava.maven.atlas.common.ref.ProjectVersionRef;
import org.commonjava.maven.atlas.effective.ref.EProjectKey;
import org.commonjava.maven.atlas.effective.util.RelationshipUtils;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class EProjectKeySer
    implements JsonSerializer<EProjectKey>, JsonDeserializer<EProjectKey>
{

    @Override
    public EProjectKey deserialize( final JsonElement src, final Type type, final JsonDeserializationContext ctx )
        throws JsonParseException
    {
        final JsonObject obj = src.getAsJsonObject();
        final JsonElement pv = obj.get( SerializationConstants.PROJECT_VERSION );
        if ( pv == null )
        {
            throw new JsonParseException( "Invalid graph key: Missing ProjectVersionRef element '"
                + SerializationConstants.PROJECT_VERSION + "'" );
        }

        final ProjectVersionRef ref = ctx.deserialize( pv, ProjectVersionRef.class );

        final JsonElement u = obj.get( SerializationConstants.SOURCE_URI );

        URI uri;
        try
        {
            uri = u == null ? RelationshipUtils.UNKNOWN_SOURCE_URI : new URI( u.getAsString() );
        }
        catch ( final URISyntaxException e )
        {
            throw new JsonParseException( "Invalid source-uri: '" + u.getAsString() + "': " + e.getMessage(), e );
        }

        return new EProjectKey( uri, ref );
    }

    @Override
    public JsonElement serialize( final EProjectKey src, final Type type, final JsonSerializationContext ctx )
    {
        final JsonObject obj = new JsonObject();
        obj.add( SerializationConstants.PROJECT_VERSION, ctx.serialize( src.getProject() ) );
        obj.addProperty( SerializationConstants.SOURCE_URI, src.getSource()
                                                               .toString() );

        return obj;
    }

}
