package org.commonjava.tensor.io.json;

import org.commonjava.maven.atlas.common.ref.ArtifactRef;
import org.commonjava.maven.atlas.common.ref.ProjectRef;
import org.commonjava.maven.atlas.common.ref.ProjectVersionRef;
import org.commonjava.maven.atlas.common.version.SingleVersion;
import org.commonjava.maven.atlas.effective.EGraphManager;
import org.commonjava.maven.atlas.effective.EProjectCycle;
import org.commonjava.maven.atlas.effective.EProjectDirectRelationships;
import org.commonjava.maven.atlas.effective.EProjectGraph;
import org.commonjava.maven.atlas.effective.EProjectWeb;
import org.commonjava.maven.atlas.effective.ref.EProjectKey;
import org.commonjava.maven.atlas.effective.rel.DependencyRelationship;
import org.commonjava.maven.atlas.effective.rel.ExtensionRelationship;
import org.commonjava.maven.atlas.effective.rel.ParentRelationship;
import org.commonjava.maven.atlas.effective.rel.PluginDependencyRelationship;
import org.commonjava.maven.atlas.effective.rel.PluginRelationship;
import org.commonjava.maven.atlas.effective.rel.ProjectRelationship;
import org.commonjava.tensor.data.GraphWorkspaceHolder;
import org.commonjava.web.json.ser.WebSerializationAdapter;

import com.google.gson.GsonBuilder;

public class TensorSerializationAdapter
    implements WebSerializationAdapter
{

    private final EGraphManager graphs;

    private final GraphWorkspaceHolder sessionManager;

    public TensorSerializationAdapter( final EGraphManager graphs, final GraphWorkspaceHolder sessionManager )
    {
        this.graphs = graphs;
        this.sessionManager = sessionManager;
    }

    @Override
    public void register( final GsonBuilder gson )
    {
        gson.registerTypeAdapter( ArtifactRef.class, new ArtifactRefSer() );
        gson.registerTypeAdapter( EProjectGraph.class, new EProjectGraphSer( graphs, sessionManager ) );
        gson.registerTypeAdapter( EProjectCycle.class, new EProjectCycleSer() );
        gson.registerTypeAdapter( EProjectKey.class, new EProjectKeySer() );
        gson.registerTypeAdapter( EProjectDirectRelationships.class, new EProjectRelsSer() );
        gson.registerTypeAdapter( EProjectWeb.class, new EProjectWebSer( graphs, sessionManager ) );
        gson.registerTypeAdapter( ProjectRef.class, new ProjectRefSer() );
        gson.registerTypeAdapter( ProjectRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( ParentRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( DependencyRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( PluginRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( ExtensionRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( PluginDependencyRelationship.class, new ProjectRelationshipSer() );
        gson.registerTypeAdapter( ProjectVersionRef.class, new ProjectVersionRefSer() );
        gson.registerTypeAdapter( SingleVersion.class, new SingleVersionSer() );
    }

}
